@@ -31,6 +31,13 @@ logging.getLogger("Parser").setLevel(logging.CRITICAL)
 DEFAULT_EXCLUDED_DATABASES: DatabaseFilter = {"SNOWFLAKE": None}
 
 
+SUPPORTED_OBJECT_DOMAIN_TYPES = (
+    "TABLE",
+    "VIEW",
+    "MATERIALIZED VIEW",
+)
+
+
 class SnowflakeLineageExtractor(BaseExtractor):
     """Snowflake lineage extractor"""
 
@@ -56,7 +63,7 @@ class SnowflakeLineageExtractor(BaseExtractor):
     ) -> Collection[ENTITY_TYPES]:
         assert isinstance(config, SnowflakeLineageExtractor.config_class())
 
-        logger.info("Fetching usage info from Snowflake")
+        logger.info("Fetching lineage info from Snowflake")
 
         self.account = config.account
         self.max_concurrency = config.max_concurrency
@@ -77,30 +84,12 @@ class SnowflakeLineageExtractor(BaseExtractor):
         with conn:
             cursor = conn.cursor()
 
-            # Join QUERY_HISTORY & ACCESS_HISTORY to include only queries that succeeded.
-            cursor.execute(
-                """
-                SELECT COUNT(*)
-                FROM
-                    SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q,
-                    SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
-                WHERE q.QUERY_ID = a.QUERY_ID
-                    AND q.EXECUTION_STATUS = 'SUCCESS'
-                    AND ARRAY_SIZE(a.BASE_OBJECTS_ACCESSED) > 0
-                    AND ARRAY_SIZE(a.OBJECTS_MODIFIED) > 0
-                    AND a.QUERY_START_TIME > %s
-                ORDER BY a.QUERY_START_TIME ASC
-                """,
-                (start_date,),
-            )
-            count = cursor.fetchone()[0]
-            batches = math.ceil(count / self.batch_size)
-            logger.info(f"Total {count} queries, dividing into {batches} batches")
-
-            queries = {
-                x: QueryWithParam(
-                    f"""
-                    SELECT a.BASE_OBJECTS_ACCESSED, a.OBJECTS_MODIFIED, q.QUERY_TEXT
+            if config.enable_lineage_from_history:
+                logger.info("Fetching access and query history")
+                # Join QUERY_HISTORY & ACCESS_HISTORY to include only queries that succeeded.
+                cursor.execute(
+                    """
+                    SELECT COUNT(*)
                     FROM
                         SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q,
                         SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
@@ -110,22 +99,54 @@ class SnowflakeLineageExtractor(BaseExtractor):
                         AND ARRAY_SIZE(a.OBJECTS_MODIFIED) > 0
                         AND a.QUERY_START_TIME > %s
                     ORDER BY a.QUERY_START_TIME ASC
-                    LIMIT {self.batch_size} OFFSET %s
                     """,
-                    (
-                        start_date,
-                        x * self.batch_size,
-                    ),
+                    (start_date,),
                 )
-                for x in range(batches)
-            }
-            async_execute(
-                conn,
-                queries,
-                "fetch_access_logs",
-                self.max_concurrency,
-                self._parse_access_logs,
-            )
+                count = cursor.fetchone()[0]
+                batches = math.ceil(count / self.batch_size)
+                logger.info(f"Total {count} queries, dividing into {batches} batches")
+
+                queries = {
+                    x: QueryWithParam(
+                        f"""
+                        SELECT a.BASE_OBJECTS_ACCESSED, a.OBJECTS_MODIFIED, q.QUERY_TEXT
+                        FROM
+                            SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q,
+                            SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
+                        WHERE q.QUERY_ID = a.QUERY_ID
+                            AND q.EXECUTION_STATUS = 'SUCCESS'
+                            AND ARRAY_SIZE(a.BASE_OBJECTS_ACCESSED) > 0
+                            AND ARRAY_SIZE(a.OBJECTS_MODIFIED) > 0
+                            AND a.QUERY_START_TIME > %s
+                        ORDER BY a.QUERY_START_TIME ASC
+                        LIMIT {self.batch_size} OFFSET %s
+                        """,
+                        (
+                            start_date,
+                            x * self.batch_size,
+                        ),
+                    )
+                    for x in range(batches)
+                }
+                async_execute(
+                    conn,
+                    queries,
+                    "fetch_access_logs",
+                    self.max_concurrency,
+                    self._parse_access_logs,
+                )
+
+            if config.enable_view_lineage:
+                logger.info("Fetching direct object dependencies")
+                cursor.execute(
+                    """
+                    SELECT REFERENCED_DATABASE, REFERENCED_SCHEMA, REFERENCED_OBJECT_NAME, REFERENCED_OBJECT_DOMAIN,
+                        REFERENCING_DATABASE, REFERENCING_SCHEMA, REFERENCING_OBJECT_NAME, REFERENCING_OBJECT_DOMAIN
+                    FROM SNOWFLAKE.ACCOUNT_USAGE.OBJECT_DEPENDENCIES;
+                    """
+                )
+                dependencies = cursor.fetchall()
+                self._parse_object_dependencies(dependencies)
 
         return self._datasets.values()
 
@@ -149,10 +170,9 @@ class SnowflakeLineageExtractor(BaseExtractor):
         # Extract source tables/views
         source_objects = parse_raw_as(List[AccessedObject], objects_accessed)
         for obj in source_objects:
-            if not obj.objectDomain or obj.objectDomain.upper() not in (
-                "TABLE",
-                "VIEW",
-                "MATERIALIZED VIEW",
+            if (
+                not obj.objectDomain
+                or obj.objectDomain.upper() not in SUPPORTED_OBJECT_DOMAIN_TYPES
             ):
                 continue
 
@@ -169,12 +189,15 @@ class SnowflakeLineageExtractor(BaseExtractor):
         # Assign source tables as upstream of each destination tables
         target_objects = parse_raw_as(List[AccessedObject], objects_modified)
         for obj in target_objects:
-            if not obj.objectDomain or obj.objectDomain.upper() != "TABLE":
+            if (
+                not obj.objectDomain
+                or obj.objectDomain.upper() not in SUPPORTED_OBJECT_DOMAIN_TYPES
+            ):
                 continue
 
             parts = obj.objectName.split(".")
             if len(parts) != 3:
-                logger.warn(f"Ignore invalid object name: {obj.objectName}")
+                logger.warning(f"Ignore invalid object name: {obj.objectName}")
                 continue
 
             database, schema, name = parts
@@ -194,3 +217,51 @@ class SnowflakeLineageExtractor(BaseExtractor):
             self._datasets[full_name] = Dataset(
                 logical_id=logical_id, upstream=upstream
             )
+
+    def _parse_object_dependencies(self, object_dependencies: List[Tuple]) -> None:
+        for (
+            source_db,
+            source_schema,
+            source_table,
+            source_object_domain,
+            target_db,
+            target_schema,
+            target_table,
+            target_object_domain,
+        ) in object_dependencies:
+            if (
+                not source_object_domain
+                or source_object_domain.upper() not in SUPPORTED_OBJECT_DOMAIN_TYPES
+                or not target_object_domain
+                or target_object_domain.upper() not in SUPPORTED_OBJECT_DOMAIN_TYPES
+            ):
+                continue
+
+            source_fullname = dataset_fullname(source_db, source_schema, source_table)
+            source_entity_id_str = str(
+                to_dataset_entity_id(
+                    source_fullname, DataPlatform.SNOWFLAKE, self.account
+                )
+            )
+
+            target_fullname = dataset_fullname(target_db, target_schema, target_table)
+
+            if not self.filter.include_table(target_db, target_schema, target_table):
+                logger.info(f"Excluding table {target_fullname}")
+                continue
+
+            target_logical_id = DatasetLogicalID(
+                name=target_fullname,
+                account=self.account,
+                platform=DataPlatform.SNOWFLAKE,
+            )
+
+            if target_fullname in self._datasets:
+                self._datasets[target_fullname].upstream.source_datasets.append(
+                    source_entity_id_str
+                )
+            else:
+                self._datasets[target_fullname] = Dataset(
+                    logical_id=target_logical_id,
+                    upstream=DatasetUpstream(source_datasets=[source_entity_id_str]),
+                )
