@@ -4,14 +4,6 @@ from dataclasses import dataclass, field
 from datetime import datetime
 from typing import Collection, List, Optional, Tuple
 
-from metaphor.models.crawler_run_metadata import Platform
-from metaphor.models.metadata_change_event import (
-    DataPlatform,
-    Dataset,
-    DatasetLogicalID,
-    DatasetQueryHistory,
-    QueryInfo,
-)
 from pydantic import parse_raw_as
 
 from metaphor.common.event_util import ENTITY_TYPES
@@ -20,6 +12,14 @@ from metaphor.common.filter import DatabaseFilter
 from metaphor.common.logger import get_logger
 from metaphor.common.query_history import TableQueryHistoryHeap
 from metaphor.common.utils import start_of_day
+from metaphor.models.crawler_run_metadata import Platform
+from metaphor.models.metadata_change_event import (
+    DataPlatform,
+    Dataset,
+    DatasetLogicalID,
+    DatasetQueryHistory,
+    QueryInfo,
+)
 from metaphor.snowflake.accessed_object import AccessedObject
 from metaphor.snowflake.auth import connect
 from metaphor.snowflake.query.config import SnowflakeQueryRunConfig
@@ -96,7 +96,7 @@ class SnowflakeQueryExtractor(BaseExtractor):
             queries = {
                 x: QueryWithParam(
                     f"""
-                    SELECT START_TIME, QUERY_TEXT, q.USER_NAME, DIRECT_OBJECTS_ACCESSED
+                    SELECT START_TIME, QUERY_TEXT, q.USER_NAME, DIRECT_OBJECTS_ACCESSED, TOTAL_ELAPSED_TIME
                     FROM SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q
                     JOIN SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
                       ON a.QUERY_ID = q.QUERY_ID
@@ -135,8 +135,16 @@ class SnowflakeQueryExtractor(BaseExtractor):
         self, batch_number: str, query_history: List[Tuple]
     ) -> None:
         logger.info(f"access logs batch #{batch_number}")
-        for start_time, query_text, username, accessed_objects in query_history:
-            self._parse_access_log(start_time, username, query_text, accessed_objects)
+        for (
+            start_time,
+            query_text,
+            username,
+            accessed_objects,
+            total_elapsed_time,
+        ) in query_history:
+            self._parse_access_log(
+                start_time, username, query_text, accessed_objects, total_elapsed_time
+            )
 
     def _parse_access_log(
         self,
@@ -144,6 +152,7 @@ class SnowflakeQueryExtractor(BaseExtractor):
         username: str,
         query_text: str,
         accessed_objects: str,
+        total_elapsed_time: float,
     ):
         try:
             objects = parse_raw_as(List[AccessedObject], accessed_objects)
@@ -167,7 +176,13 @@ class SnowflakeQueryExtractor(BaseExtractor):
                     logger.debug(f"Ignore {table_name} due to filter config")
                     continue
                 self._table_queries.store_recent_query(
-                    table_name, start_time, query_text, username
+                    table_name,
+                    QueryInfo(
+                        elapsed_time=total_elapsed_time / 1000,
+                        issued_at=start_time,
+                        issued_by=username,
+                        query=query_text,
+                    ),
                 )
         except Exception:
             logger.exception(f"access log error, objects: {accessed_objects}")
